//
// ddisp.cpp
// 
// Simple application for DCAM capture
//
//
// Copyright 2001 by Videre Design 
//


//
// top-level loop for grabbing and displaying
// 

#include "dcam.h"
#include "timer.h"
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>

#include "flwin.h"

#ifdef WIN32
#include <windows.h>
#include <mmsystem.h>
#include "resource.h"

HINSTANCE hInstance;		/* application instance */

// returns an icon instance for the program
char *
get_icon(void)
{
  return (char *)LoadIcon(hInstance, MAKEINTRESOURCE(DCAM_ICON));
}

// OS-dependent message
void
print_msg(char *msg)
{
  MessageBox(NULL, msg, "Error", MB_OK);  
}

// reference to the main program
int main(int argc, char **argv);

// OS-dependent startup code
int WINAPI WinMain (HINSTANCE hInst, HINSTANCE hPrevInstance,
		    LPSTR lpszCmdLine, int nCmdShow)
{
  hInstance = hInst;
  main(0,NULL);
  return 0;
}

#else
// Linux definitions
char *get_icon(void)
{
  return NULL;
} 

void print_msg(char *msg)
{
  printf(msg);
}
#endif


Fl_Window *mainw;		// main FLTK application window
dcamWin *win;			// dcam display window
dCamera *dcam;			// camera object

char *apparg = "dcam";

int 
main(int argc, char **argv)
{
  Fl::visual(FL_RGB);
  mainw = new Fl_Window(327, 276, "Videre Design DCAM Capture (2.1)");
  win = new dcamWin(3, 23, 320, 240);
  mainw->icon(get_icon());
  mainw->show(1,&apparg);	// need args to get icon correct

  dSysInit();                   // sets global var dSys
  if (dSys == NULL)		// oops, problems initializing
    {
      print_msg("Can't initialize IEEE 1394 bus");
      exit(-1);
    }

  dcam = dSys->InitCamera(0);	// get first one found
  if (!dcam)
    {
      print_msg("Can't find a camera");
      exit(-1);
    }


  // set format to 640x480, RGB color, 15 fps
  if (!dcam->SetFormat(S_640x480, S_RGB24, S_15))
    {
      print_msg("Can't set format");
      exit(-1);
    }

  // start video streaming
  if (!dcam->Start())
    {
      print_msg("Can't start video streaming");
      exit(-1);
    }

  while (1)
    {
      if (!Fl::check())
	return 0;		// we're done here

      if (!dcam)		// check if there's a camera
	return 0;

      unsigned char *im;
      if (dcam->GetImage(&im, 500))
	win->DrawImage(im, dcam);
  } 
}

