//
// sync2.cpp
// 
// Simple syncing application for DCAM capture
// Open 2 cameras, syncs and displays them
//
// Copyright 2001 by Videre Design 
//


//
// top-level loop for grabbing and displaying
// 

#include "dcam.h"
#include "timer.h"
#ifdef WIN32
#include "direct.h"
#else
#include <unistd.h>
#endif
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>

#include "flwin.h"

#ifdef WIN32
#include <windows.h>
#include <windowsx.h>
#include "resource.h"

HINSTANCE hInstance;		/* application instance */

// returns an icon instance for the program
char *
get_icon(void)
{
  return (char *)LoadIcon(hInstance, MAKEINTRESOURCE(DCAM_ICON));
}

// reference to the main program
int main(int argc, char **argv);

// OS-dependent startup code
int WINAPI WinMain (HINSTANCE hInst, HINSTANCE hPrevInstance,
		    LPSTR lpszCmdLine, int nCmdShow)
{
  hInstance = hInst;
  main(0,NULL);
  return 0;
}

#else
// Linux definitions
char *get_icon(void)
{
  return NULL;
} 
#endif


//
// Debug browser for messages
//

Fl_Browser *debugData;
Fl_Window *debugWin;
static char mess[256];

void
capDebug(char *str)
{
  if (debugWin)
    {
      if (str)
	debugData->add(str);  
    }
}


void
print_msg(char *msg, ...)
{
  va_list ptr;
  va_start(ptr,msg);
  vprintf(msg,ptr);
  printf("\n");
  vsprintf(mess,msg,ptr);
  if (debugWin)
    debugData->add(mess);  
  va_end(ptr);
}

void
wait_end()
{
  while (1)
    {
      if (!Fl::check())
	exit(-1);		// we're done here
    }
}

Fl_Window *mainw;		// main FLTK application window
dcamWin *win, *win2;		// dcam display window
dCamera *dcam, *dcam2;		// camera object

char *apparg = "dcam";


int
main(int argc, char **argv)
{
  Fl::visual(FL_RGB);
  mainw = new Fl_Window(650, 276, "Videre Design DCAM Capture (2.0)");
  win = new dcamWin(3, 23, 320, 240);
  win2 = new dcamWin(327, 23, 320, 240);
  mainw->icon(get_icon());
  mainw->show(1,&apparg);	// need args to get icon correct
  debugWin = new Fl_Window(100,100,500,400, "DCAM Debugging Window");
  debugData= new Fl_Browser(0,0,500,400);
  debugWin->end();
  debugData->add("Debugging Window");
  debugWin->show();
  dcamDebug = capDebug;		// write debug message from the driver

  dSys = dSysInit();
  if (dSys == NULL)		// oops, problems initializing
    {
      print_msg("Can't initialize IEEE 1394 bus");
      wait_end();
    }
  print_msg("IEEE 1394 bus initialized");
  print_msg("Found %d cameras", dSys->NumCameras());

  dcam = dSys->InitCamera(0);	// get first one
  if (!dcam)
    {
      print_msg("Can't find a camera");
      wait_end();
    }
  print_msg("Camera 0 initialized");


  if (!dcam->SetFormat(S_640x480, S_RGB24, S_15))
//  if (!dcam->SetFormat(S_640x480, S_RGB24, S_60))
    {
      print_msg("Can't set format on first camera");
      wait_end();
    }
  print_msg("Camera format set to 640x480, RGB24, 15 Hz");

  dcam2 = dSys->InitCamera(1);	// get second one
  if (!dcam2)
    {
      print_msg("Can't find second camera");
      wait_end();
    }
  print_msg("Camera 1 initialized");  

  if (!dcam2->SetFormat(S_640x480, S_RGB24, S_15))
    //  if (!dcam2->SetFormat(S_640x480, S_RGB24, S_60))
    {
      print_msg("Can't set format on second camera");
      wait_end();
    }
  print_msg("Camera format set to 640x480, RGB24, 15 Hz");


  // start video streaming
  if (!dcam->Start())
    {
      print_msg("Can't start video streaming");
      wait_end();
    }
  print_msg("Camera 0 video streaming started");

  if (!dcam2->Start())
    {
      print_msg("Can't start 2nd video streaming");
      wait_end();
    }
  print_msg("Camera 1 video streaming started");

  int f0, f1;
  unsigned long t0, t1, last0 = 0, last1 = 0;
  while (1)
    {
      if (!Fl::check())
      {
        dcam->Stop();
        dcam2->Stop();
	return 0;		// we're done here
      }

      if (!dcam && !dcam2)	// check if there's a camera
				continue;

      unsigned char *im, *im2;
      if (dcam->GetImage(&im, 500, &f0, &t0))
      {
				win->DrawImage(im, dcam);
//        print_msg("Cam 0: %d %d %d", f0, t0-last0, t0);        
        last0 = t0;
      }

      if (dcam2->GetImage(&im2, 500, &f1, &t1))
      {
				win2->DrawImage(im2, dcam);
//        print_msg("Cam 1: %d %d %d", f1, t1-last1, t1);        
        last1 = t1;
      }

    }
}

