//
// flwin.h
// 
// display window class definitions
//
//
// Copyright 2000 by Videre Design 
//

#include "dcam.h"

#ifndef FLWIN_H
#define FLWIN_H

#define index xx
#undef index
#include "FL/Fl.H"
#include "FL/Fl_Group.H"
#include "FL/Fl_Window.H"
#include "FL/fl_draw.H"
#include "FL/Fl_Double_Window.H"
#include "FL/Fl_Browser.H"
#include "FL/Fl_Select_Browser.H"
#include "FL/Fl_Text_Display.H"
#include "FL/Fl_Text_Buffer.H"
#include "FL/Fl_Menu_Bar.H"
#include "FL/Fl_File_Chooser.H"
#include "fl_value.h"
#include <stdarg.h>

class dcamWin : public Fl_Double_Window
{
public:
  dcamWin(int x, int y, int h, int w);
  void DrawImage(unsigned char *im, dCamera *);
  void ClearImage();
  void draw();                  // drawing routine 

  // if we want to do something with mouse clicks...
  void ButtonHandler(int (*fn)(int, int, int, int, dcamWin *))
    { bhandler = fn; }

  int dwidth, dheight;          // display window size, doesn't change
  int width, height;            // displayed image size
  int xoff, yoff;               // displayed image offset withing display window

  // set up overlay function
  void DrawOverlayFn(void (*fn)(dcamWin *, void *ovarg), void *ova = NULL)
    { ofn = fn; ovArg = ova; };

  // display gamma value
  double displayGamma;

 private: 
  void DisplayImage(unsigned char *im, int w, int h, int ls, 
		    int type = S_MONO, int s = 1, double gam = 1.0, 
		    int x = 0, int y = 0);  
  void drawit();
  int skipw, linesize;          // pixel decimation, line size of original image 
  unsigned char *pixelData;
  int pixelType;
  double oldgamma;		// for gamma conversion
  unsigned char gamtab[256];

  // to check key and mouse events
  int handle(int);              // has to be IMPORTed so subclasses can see it
  int (*bhandler)(int, int, int, int, dcamWin *);

 private:
  void (* ofn)(dcamWin *, void *ovarg); // set this to be the overlay drawing fn
  void *ovArg;                  // stored arg to overlay fn
};


class dcamDebugWin : public Fl_Window
{
public:
  dcamDebugWin(int, int, int, int, char *name = NULL);
  void Print(char *str, ...);
  void Bottom();
  Fl_Text_Display *dwin;
  Fl_Text_Buffer *dbuf;
private:
  char outp[256];
};

extern "C" {
int fltk_check(void);
}


#endif  // FLWIN_H
